#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h> 
#include <string.h>
#include <unistd.h>

#define FLIP4(n)  (  (n & 0xff000000) >> 24		\
		     | (n & 0x00ff0000) >> 8		\
		     | (n & 0x0000ff00) << 8		\
		     | (n & 0x000000ff) << 24  )


/*************************************************************
 * java_fser_
 * c program to link with fortran 90
 * This program writes the x and fx values to file.
 *
 * input parameters
 *   name - name of the file
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 ************************************************************/

int java_fser_(name, time, x, fx, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
{
   int size, i;
   FILE *fopen(), *fp;

   if ((fp = fopen(name, "a+b")) == NULL) {
      printf("Error: can't open output file ...\n");
      return 1;
   }

   /* first write time and nx */
   fwrite(time, sizeof(double), 1, fp);
   fwrite(nx, sizeof(int), 1, fp);

   size = *nx;
   
   /* write x and fx value */
   for (i=0; i<size; i++) {
      fwrite(&x[i], sizeof(double), 1, fp);
      fwrite(&fx[i], sizeof(double), 1, fp);
   }
   
   fclose(fp);
   return 0;
}


/*************************************************************
 * java_ser_
 * c program to link with fortran 90
 * This program writes the x and fx values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_ser_(name, time, x, fx, nx) 
  char *name;
  double *time;
  double x[];
  double fx[];
  int *nx;
{
   int sockfd;
   int size, name_length, i;

   int command = 10;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);

   size = *nx; 
   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, &size, sizeof(int));

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &x[i], sizeof(double));
      write(sockfd, &fx[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   fx   - fx value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_mser_(name, nt, time, x, fx, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double fx[];
  int nx[];
{
   int sockfd;
   int ntsize, nxsize, name_length, i, j, index;

   int command = 12;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      nxsize = nx[i];     
      write(sockfd, &nxsize, sizeof(int));
      for (j=0; j<nxsize; j++) {
         write(sockfd, &x[index], sizeof(double));
         write(sockfd, &fx[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_fser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to file.
 *
 * input parameters
 *   name - name of the file
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   fxy  - fxy value array
 *   nx   - size of x, y and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 ************************************************************/

int java_fser2d_(name, time, x, y, fxy, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
{
   int size, i;
   FILE *fopen(), *fp;

   if ((fp = fopen(name, "a+b")) == NULL) {
      printf("Error: can't open output file ...\n");
      return 1;
   }

   /* first write time and nx */
   fwrite(time, sizeof(double), 1, fp);
   fwrite(nx, sizeof(int), 1, fp);

   size = *nx;
   
   /* write x and fx value */
   for (i=0; i<size; i++) {
      fwrite(&x[i], sizeof(double), 1, fp);
      fwrite(&y[i], sizeof(double), 1, fp);
      fwrite(&fxy[i], sizeof(double), 1, fp);
   }
   
   fclose(fp);
   return 0;
}


/*************************************************************
 * java_ser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   fxy  - fxy value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_ser2d_(name, time, x, y, fxy, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double fxy[];
  int *nx;
{
   int sockfd;
   int size, name_length, i;

   int command = 15;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   size = *nx; 
   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, &size, sizeof(int));

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &x[i], sizeof(double));
      write(sockfd, &y[i], sizeof(double));
      write(sockfd, &fxy[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_mser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   y    - y value array
 *   fxy   - fxy value array
 *   nx   - size of x and fx array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_mser2d_(name, nt, time, x, y, fxy, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double y[];
  double fxy[];
  int nx[];
{
   int sockfd;
   int ntsize, nxsize, name_length, i, j, index;

   int command = 16;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      nxsize = nx[i];     
      write(sockfd, &nxsize, sizeof(int));
      for (j=0; j<nxsize; j++) {
         write(sockfd, &x[index], sizeof(double));
         write(sockfd, &y[index], sizeof(double));
         write(sockfd, &fxy[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   nx   - x array size
 *   y    - y value array
 *   ny   - y array size
 *   fxy  - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bser2d_(name, time, x, nx, y, ny, fxy) 
  char *name;
  double *time;
  double x[];
  int *nx;
  double y[];
  int *ny;
  double fxy[];
{
   int sockfd;
   int size, xsize, ysize, name_length, i;

   int command = 30;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);

   /* write time */
   write(sockfd, time, sizeof(double));

   /* send x array */
   xsize = *nx; 
   write(sockfd, &size, sizeof(int));
   for (i=0; i<xsize; i++) 
      write(sockfd, &x[i], sizeof(double));

   /* send y array */
   ysize = *ny; 
   write(sockfd, &ysize, sizeof(int));
   for (i=0; i<size; i++) 
      write(sockfd, &y[i], sizeof(double));

   /* send fxy array */
   size = xsize * ysize;
   write(sockfd, &size, sizeof(int));
   for (i=0; i<size; i++) 
      write(sockfd, &fxy[i], sizeof(double));

   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bmser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   nx   - size of x array
 *   y    - y value array
 *   ny   - size of y array 
 *   fxy   - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bmser2d_(name, nt, time, x, nx, y, ny, fxy) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double nx[];
  double y[];
  double ny[];
  double fxy[];
{
   int sockfd;
   int ntsize, xsize, ysize, size, name_length, i, j, index, xindex, yindex;

   int command = 31;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;
   xindex = 0;
   yindex = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      xsize = nx[i];     
      write(sockfd, &xsize, sizeof(int));
      for (j=0; j<xsize; j++) {
         write(sockfd, &x[xindex], sizeof(double));
         xindex++;
      }

      ysize = ny[i];     
      write(sockfd, &ysize, sizeof(int));
      for (j=0; j<ysize; j++) {
         write(sockfd, &y[yindex], sizeof(double));
         yindex++;
      }

      size = xsize * ysize;
      for (j=0; j<size; j++) {
         write(sockfd, &fxy[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}

/*************************************************************
 * java_bbser2d_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   xmin    - x min value
 *   xmax    - x max value
 *   nx      - number of x points 
 *   ymin    - y min value
 *   ymax    - y max value
 *   ny      - number of y points 
 *   fxy     - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bbser2d_(name, time, xmin, xmax, nx, ymin, ymax, ny, fxy) 
  char *name;
  double *time;
  double *xmin;
  double *xmax;
  int *nx;
  double *ymin;
  double *ymax;
  int *ny;
  double fxy[];
{
   int sockfd;
   int size, xsize, ysize, name_length, i;

   int command = 25;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);

   xsize = *nx;
   ysize = *ny;
   size = xsize * ysize;
   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, nx, sizeof(int));
   write(sockfd, ny, sizeof(int));
   write(sockfd, xmin, sizeof(double));
   write(sockfd, xmax, sizeof(double));
   write(sockfd, ymin, sizeof(double));
   write(sockfd, ymax, sizeof(double));

   /* write fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &fxy[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bbmser2d_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   xmin    - x min value array
 *   xmax    - x max value array
 *   nx    -   nx value array
 *   ymin    - y min value array
 *   ymax    - y max value array
 *   ny    -   ny value array
 *   fxy   - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bbmser2d_(name, nt, time, xmin, xmax, nx, ymin, ymax, ny, fxy) 
  char *name;
  int *nt;
  double time[];
  double xmin[];
  double xmax[];
  int nx[];
  double ymin[];
  double ymax[];
  int ny[];
  double fxy[];

{
   int sockfd;
   int ntsize, nxsize, nysize, size, name_length, i, j, index;

   int command = 26;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      nxsize = nx[i];   
      nysize = ny[i];  
      write(sockfd, &nxsize, sizeof(int));
      write(sockfd, &nysize, sizeof(int));
      write(sockfd, &xmin[i], sizeof(double));
      write(sockfd, &xmax[i], sizeof(double));
      write(sockfd, &ymin[i], sizeof(double));
      write(sockfd, &ymax[i], sizeof(double));
     
      size = nxsize * nysize;
      for (j=0; j<size; j++) {
         write(sockfd, &fxy[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_vser_
 * c program to link with fortran 90
 * This program writes the vector data to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   x    - x value array
 *   y    - y value array
 *   r  - r value array
 *   t  - r value array
 *   nx   - size of x, y, r, t array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_vser_(name, time, x, y, r, t, nx) 
  char *name;
  double *time;
  double x[];
  double y[];
  double r[];
  double t[];
  int *nx;
{
   int sockfd;
   int size, name_length, i;

   int command = 22;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   size = *nx; 
   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, &size, sizeof(int));

   /* write x and fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &x[i], sizeof(double));
      write(sockfd, &y[i], sizeof(double));
      write(sockfd, &r[i], sizeof(double));
      write(sockfd, &t[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_vmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x and fx values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   x    - x value array
 *   y    - y value array
 *   r  - r value array
 *   t  - t value array
 *   nx   - size of x, y, r, t array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_vmser_(name, nt, time, x, y, r, t, nx) 
  char *name;
  int *nt;
  double time[];
  double x[];
  double y[];
  double r[];
  double t[];
  int nx[];
{
   int sockfd;
   int ntsize, nxsize, name_length, i, j, index;

   int command = 27;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);


   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      nxsize = nx[i];     
      write(sockfd, &nxsize, sizeof(int));
      for (j=0; j<nxsize; j++) {
         write(sockfd, &x[index], sizeof(double));
         write(sockfd, &y[index], sizeof(double));
         write(sockfd, &r[index], sizeof(double));
         write(sockfd, &t[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}




/*************************************************************
 * java_bbser_
 * c program to link with fortran 90
 * This program writes the x, y and fxy values to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   time - time value
 *   xmin    - x min value
 *   xmax    - x max value
 *   nx      - number of x points 
 *   fx     - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bbser_(name, time, xmin, xmax, nx, fx) 
  char *name;
  double *time;
  double *xmin;
  double *xmax;
  int *nx;
  double fx[];
{
   int sockfd;
   int size, name_length, i;

   int command = 28;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);

   size = *nx;

   /* write time and nx */
   write(sockfd, time, sizeof(double));
   write(sockfd, nx, sizeof(int));
   write(sockfd, xmin, sizeof(double));
   write(sockfd, xmax, sizeof(double));


   /* write fx value */
   for (i=0; i<size; i++) {
      write(sockfd, &fx[i], sizeof(double));
   }
   
   close(sockfd);
   return 0;
}


/*************************************************************
 * java_bbmser_
 * c program to link with fortran 90
 * This program writes whole set of time data with the x, y and fxy values
 * to socket.
 *
 * input parameters
 *   name - name of the file, it will be used fore title
 *   nt   - number of time
 *   time - time value array
 *   xmin    - x min value array
 *   xmax    - x max value array
 *   nx    -   nx value array
 *   fx   - fxy value array
 *
 * return value
 *   0 if success
 *   1 otherwise
 *
 * It will read server host name and port from JSERHOST and
 * JSERPORT environment.
 ************************************************************/
int java_bbmser_(name, nt, time, xmin, xmax, nx, fx) 
  char *name;
  int *nt;
  double time[];
  double xmin[];
  double xmax[];
  int nx[];
  double fx[];

{
   int sockfd;
   int ntsize, size, name_length, i, j, index;

   int command = 26;

   sockfd = initialize();
   if (sockfd == -1)
     exit(1);

   /* first send command */
   write(sockfd, &command, sizeof(int));

   
   /* send title information */
   name_length = strlen(name);
   write(sockfd, &name_length, sizeof(int));
   write(sockfd, name, name_length);


   ntsize = *nt; 
   /* send nt */
   write(sockfd, &ntsize, sizeof(int));

   /* write time data */
   index = 0;

   for (i=0; i<ntsize; i++) {
      write(sockfd, &time[i], sizeof(double));

      size = nx[i];   
      write(sockfd, &size, sizeof(int));
      write(sockfd, &xmin[i], sizeof(double));
      write(sockfd, &xmax[i], sizeof(double));
     
      for (j=0; j<size; j++) {
         write(sockfd, &fx[index], sizeof(double));
         index++;
      }
   }
   
   close(sockfd);
   return 0;
}


/******************************************************************
 * initialize the socket
 * return socket descriptor. If failed return -1
 ******************************************************************/
int initialize()
{
   int sockfd;

   char *hostname;
   char *rawport;
   int port;

   struct  hostent *h;
   struct sockaddr_in serv_addr;

   /** 
    * first read system environment JSERHOST and JSERPORT
    */
   hostname = getenv("JSERHOST");
   rawport = getenv("JSERPORT");

   if (hostname == NULL) {
      printf("Unknown server host name error ...\n");
      return -1;
   }
   
   if (rawport == NULL) 
      port = 7654;
   else
      port = atoi(rawport);

   if ((h=gethostbyname(hostname)) == NULL) {  /* get the host info */
      herror("gethostbyname");
      return -1;
   }

   /*
    * fill in the structure "serv_addr" with the address 
    * of the server that we want to connect with 
    */
   bzero((char *) &serv_addr, sizeof(serv_addr));
   serv_addr.sin_family = AF_INET;
   serv_addr.sin_addr.s_addr = 
                    inet_addr(inet_ntoa(*((struct in_addr *)h->h_addr)));
   serv_addr.sin_port = htons(port);

   printf("Host name  : %s\n", h->h_name);
   printf("IP Address : %s\n",inet_ntoa(*((struct in_addr *)h->h_addr)));

   /*
    * open a TCP socket (an Internet stream socket).
    */
   if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      printf("Can't open stream socket.\n");
      return -1;
   }

   /*
    * connect to server
    */
   if (connect(sockfd, (struct sockaddr *) &serv_addr, 
                                    sizeof(serv_addr)) < 0) {
      printf("Can't connect to server.\n");
      return -1;
   }

   return sockfd;

}


/*** receive_data *****************************************************
   Receive a block of data by repeatedly reading from a socket.  This
   function will block the process if data is not available.
   Input:  socket - socket to read from
            buffer - pointer to data buffer
           bytes - how many bytes to read into buffer
   Return:  number of bytes received.  This will always either be
            the same as the bytes requested or 0 if the connection
            was lost.
**********************************************************************/
int receive_data( socket, buffer, bytes )
int socket;
char *buffer;
int bytes;
{
   int sofar, remaining, len;

   sofar = 0;
   remaining = bytes;
   do {
      len = read( socket, buffer + sofar, remaining);
      if (len<=0) return 0;
      sofar += len;
      remaining -= len;
   } while (remaining>0);
   return bytes;
}




/*** send_data ********************************************************
   Send a block of data by repeatedly writing to a socket.
   Input:  socket - socket to write to
           buffer - pointer to data to send
           bytes - number of bytes to send from buffer
**********************************************************************/
void send_data( socket, buffer, bytes )
int socket;
char *buffer;
int bytes;
{
   int sofar, remaining, len;

   sofar = 0;
   remaining = bytes;
   do {
      len = write( socket, buffer + sofar, remaining);
      sofar += len;
      remaining -= len;
   } while (remaining>0);

}




/*** receive_int ******************************************************
   Receive a 32-bit integer through a socket.  This function will
   block if the data is not available.
   Input:  socket - the socket to receive the integer from.
           i - pointer to integer to put result into.
   Output:  i will have the value received
   Return:  0 if the connection was lost
            non-zero if successful
**********************************************************************/
int receive_int( socket, i )
int socket;
int *i;
{
   return receive_data( socket, i, sizeof(int) );
}




/*** send_int *********************************************************
   Send a 32-bit integer through a socket.
   Input:  socket - the socket to send on.
           i - the integer to send
   Output:  none.
**********************************************************************/
void send_int( socket, i )
int socket;
int i;
{
   send_data( socket, &i, sizeof(int) );
}





/*** receive_float ****************************************************
   Receive a 32-bit float through a socket.  This function will
   block if the data is not available.
   Input:  socket - the socket to receive the integer from.
           x - pointer to float to put result into.
   Output:  i will have the value received
   Return:  0 if the connection was lost
            non-zero if successful
**********************************************************************/
int receive_float( socket, x )
int socket;
float *x;
{
   return receive_data( socket, x, sizeof(float) );
}





/*** send_float *********************************************************
   Send a 32-bit float through a socket.
   Input:  socket - the socket to send on.
           x - the float to send
   Output:  none.
**********************************************************************/
void send_float( socket, x )
int socket;
float x;
{
   float y;

   y = (float) x;
   send_data( socket, &y, sizeof(float) );
}




/*** receive_str ******************************************************
   Receive a character string.  This function will block if the data
   is not available.
   Input:  socket - the socket to receive the string from.
           str - pointer to char array to put result into.  It is
                 assumed to be of sufficient size.
   Output:  str will have the string received
   Return:  0 if the connection was lost
            non-zero if successful
**********************************************************************/
int receive_str( socket, str )
int socket;
char str[];
{
   int len;

   if ( receive_int(socket, &len) ) {
      if ( receive_data(socket, str, len)==len ) {
	 str[len] = '\0';
         return 1;
      }
   }
   return 0;
}




/*** send_str *********************************************************
   Send a character string through a socket.
   Input:  socket - the socket to send on.
           str - the string to send
   Output:  none.
**********************************************************************/
void send_str( socket, str )
int socket;
char str[];
{
   int len;

   len = strlen( str );
   send_int( socket, len );
   send_data( socket, str, len );
}




main() {

   int sockfd;
   int i, len;
   float f;
   int command = 101;
   float x = 10.0;
   double d=256, y;
   char *sendstr = "test string ...";
   char revstr[500];
   
   sockfd = initialize();
   if (sockfd == -1)
     exit(1);


   /* test int communication */
   send_int(sockfd, command);
   receive_int(sockfd, &i);

printf("\nreceived int (expected 100) ====>  %d\n", i);

   /* test float communication */
   send_float(sockfd, x);
   receive_float(sockfd, &f);
printf("received float (expected 100) ====>  %f\n", f);

   /* test float communication */
   write(sockfd, &d, sizeof(double));
   receive_data(sockfd, &y, sizeof(double) );

printf("received double (expected 255) ====>  %f\n", y);


   /* test string communication */
   i = strlen(sendstr);
   write(sockfd, &i, sizeof(int));
   write(sockfd, sendstr, i);
   receive_str(sockfd, revstr[0]); 
printf("received string (expected \"Hey client 100\"):  %s\n", revstr);
printf("received string (expected \"Hey client 100\"):  %s\n", &revstr);
printf("received string (expected \"Hey client 100\"):  %s\n", *revstr);
   close(sockfd);
   return 0;

}












